--[[

Idea: Provide filter / replacement service for commonly used acronymns in game.
	links when moused over in default chat frames should show the game tooltip
	with the description provided.

Addon Name: AcronymGuide (AG)


]]--


local function Debug(self, ...)
	local str = string.join(", ", tostringall(...) )
	str = str:gsub(":,", ":"):gsub("=,", "=")
	--[===[@alpha@
	if true then
		ChatFrame5:AddMessage("|cff6666ffAcronymGuide:|r "..str)
	end
	--@end-alpha@]===]
	return str
end

local tinsert, select, assert, tostringall = table.insert, select, assert, tostringall



AcronymGuide = LibStub("AceAddon-3.0"):NewAddon("AcronymGuide")
local addon = AcronymGuide
addon.Debug = Debug

local localizedName = "|cff6666ffAcronymGuide:|r"	--L["AcronymGuide"]

function addon:OnInit()
	self.db = LibStub("AceDB-3.0"):New("TG1")
end

		
function addon:OnEnable()
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", self.DA_Main_Filter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY", self.DA_Main_Filter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_GUILD", self.DA_Main_Filter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY", self.DA_Main_Filter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", self.DA_Main_Filter)
	
	self:HookTooltip(GameTooltip)
	self:HookTooltip(ItemRefTooltip)
end
--------------------
--	Link Registry Stuff
--------------------

local regex_link, uid_desc, uid_linkText, reg_index = {}, {}, {}, {}
addon.regex_link, addon.uid_desc, addon.uid_linkText, addon.reg_index = regex_link, uid_desc, uid_linkText, reg_index

local fix = "%s"
function addon:AddDefinition(def)
	uid, regex, linkText = tostringall( unpack(def) )
	tremove(def, 1)
	tremove(def, 1)
	tremove(def, 1)
	--Sanity Batterie 1: is any of the data bullshit?
	assert( (uid ~= "nil") or (uid ~= ""))
	assert( (regex ~= "nil") or (regex ~= ""))
	assert( (linkText ~= "nil") or (linkText ~= ""))
	regex = fix..regex..fix
	
	if regex_link[regex] then
		self:Debug("Duplicate Regex:", regex)
		--self:Debug("Existing", regex, entryText[1])	--fix
		--self:Debug("New:", uid, regex, entryText[1]	--fix
		return
	end
	
	tinsert(reg_index, regex)
	regex_link[regex] = self:GetLink(uid, linkText)
	uid_desc[uid] = def
	uid_linkText[uid] = linkText
	return true
end

local Master_Link_Format = "|cff6666ff|HAG:%s~|h[%s]|h|r"
local Unknown_Link = "|cff6666ff|HDAG:Unknown~|h[?]|h|r"
local reverse_regex = "AG:(.-)~"

addon.Master_Link_Format = Master_Link_Format
addon.Unknown_Link = Unknown_Link
addon.reverse_regex = reverse_regex


function addon:GetLink(uid, linkText)
	assert( (uid) and ( type(uid) == "string") )
	if not linkText then
		if uid_linkText[uid] then
			return Master_Link_Format:format(uid, uid_linkText[uid] )
		else
			return self.Unknown_Link
		end
	end
	return Master_Link_Format:format(uid, linkText)
end

---Example and Default Unknown Link key

do
	local def = {
	{"Unknown", "^%?", "?", ":Unknown Link:", "If you are seeing this then there is an error.", "or you are in Debug Mode :)" }
	}
	for i = 1, #def do
		addon:AddDefinition(def[i])
	end
end

addon.Unknown_Link = addon:GetLink("Unknown")

--------------------
---	Filter Stuff
--------------------
--[[
	Using the method from WIM's Emoticons.lua file where it replaces the links with a \001\004 + table index.
	i believe i can expand on that by just replacing the things i wants links for wtih the same method and adding to said table.
]]--

do
	local tempLinks = {}
	
	function addon.DA_Main_Filter(daMsg)
		daMsg = " "..tostring(daMsg).." "
		if (daMsg == "nil") or (daMsg == "") then
			return false, daMsg
		end
	
		-- We're gonna borrow some methods from WIM's Emoticons.lua file :)
		-- Like not disturbing links n such.
		local result
		wipe(tempLinks)

		repeat			---Taken from WIM\Emoticons.lua:91-96 ; Adapted for Context
			daMsg, result = string.gsub(daMsg, "(|c%x+|H[^|]+|h[^|]+|h|r)", function(theLink)
				table.insert(tempLinks, theLink);
				return "\001\004"..#tempLinks;
		end, 1);
		until result == 0;	--End

		---Going to do our link stuff here
		for i=1, #reg_index do
			daMsg, result = daMsg:gsub(reg_index[i], function(segment)
				table.insert(tempLinks, regex_link[ reg_index[i] ] )
				return "\001\004"..#tempLinks;
			end )
		end
	
		--Cleanup
		--Put the existing links back
		for i=1, #tempLinks do	--Again Taken from WIM\Emoticons.lua:109-111; Adapted for Context
			daMsg = string.gsub(daMsg, "\001\004"..i.."", " "..tempLinks[i].." ");
		end
		wipe(tempLinks)
	
		----------------
		addon.cf1 = false
		return false, daMsg:trim()
	end

end


--------------------
---	Tooltip Hooks
--------------------

--[[
	Make a generator function that other addons can use.
	hook the chatframes so that we have hoverlinks.	--gonna grab it from tekKompare, if your going to steal, steal from the best :)
]]--
do
	local function colorCapLetterGreen(str)
		if str == str:upper() then
			return "|cff1eff00"..tostring(str):upper().."|r"
		else
			return str
		end
	end
	local function MakeItATitleWithColor(title)
		title = title:gsub(".", colorCapLetterGreen)
		return title
	end

	local hooked_tooltips = {}

	local function new_SetHyperlink(tip, id, txt, ...)
		if not hooked_tooltips[tip] then
			print("AG Tooltip Hooked called for", tip.GetName and tip.GetName() or tip, "and is not registered.", debugstack() )
		end
		if id:find(reverse_regex) then
			local _, _, uid = id:find(reverse_regex)
			--if tip.AcronymGuideID and tip.AcronymGuideID == id then
			--	--If we're parented to a chat frame then don't close.
			--	if tip:IsShown() and addon.chatFrame_onEnter[tip:GetOwner()] then return end
			--	tip.AcronymGuideID = nil
			--	tip:Hide()
			--	return
			--end
			tip:ClearLines()
			tip:AddLine(localizedName)
			local desc = uid_desc[uid] and uid_desc[uid] or uid_desc["Unknown"]
			tip:AddLine( MakeItATitleWithColor(desc[1]) )
			if #desc < 2 then return end
			for i = 2, #desc do
				tip:AddLine(desc[i])
			end
			tip.AcronymGuideID = id
			tip:Show()
			return
		end

		return hooked_tooltips[tip](tip, id, txt, ...)
	end


	function addon:HookTooltip(tip)
		if hooked_tooltips[tip] then
			return true
		end
		hooked_tooltips[tip] = tip.SetHyperlink
		tip.SetHyperlink = new_SetHyperlink	
	end
end

do	--Straight Copy and Paste from tekKompare\Hovertips.lua by Tekkub
	local cf_onEnter, cf_onLeave = {}, {}
	addon.chatFrame_onEnter, addon.chatFrame_onLeave = cf_onEnter, cf_onLeave
	local GameTooltip = GameTooltip

	local linktypes = { AG = true }	--Only Line Modified, adapted to specific needs of Addon

	local function OnHyperlinkEnter(frame, link, ...)
		local linktype = link:match("^([^:]+)")
		if linktype and linktypes[linktype] then
			GameTooltip:SetOwner(frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetHyperlink(link)
			GameTooltip:Show()
		end

		if cf_onEnter[frame] then return cf_onEnter[frame](frame, link, ...) end
	end

	local function OnHyperlinkLeave(frame, ...)
		GameTooltip:Hide()
		if cf_onLeave[frame] then return cf_onLeave[frame](frame, ...) end
	end
	local noop = function() end
	local _G = getfenv(0)
	for i=1, NUM_CHAT_WINDOWS do
		local frame = _G["ChatFrame"..i]
		cf_onEnter[frame] = frame:GetScript("OnHyperlinkEnter") or noop
		frame:SetScript("OnHyperlinkEnter", OnHyperlinkEnter)

		cf_onLeave[frame] = frame:GetScript("OnHyperlinkLeave") or noop
		frame:SetScript("OnHyperlinkLeave", OnHyperlinkLeave)
	end
end

